<?/*
Copyright (C) 2003 Media Net Link, Inc. Danville, CA 94526 USA

This file is part of AthenaRMS.

AthenaRMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

AthenaRMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/?>
<?php

require_once 'html.php';

class Problem_Type extends HTML {
  function Problem_Type () {
    global $dbi;
    $this->db = &$dbi;
  }

  function Add ($in_ary) {
    $add = array();
    foreach ($in_ary as $k => $v) {
      $add[strtoupper($k)] = $v;
    }

    if (!array_key_exists('PROBLEM_TYPE', $add) || !$add['PROBLEM_TYPE']) 
      return;

    if (!array_key_exists('CRM_ID', $add)) 
      $add['CRM_ID'] = $this->Current_CRM ();

    if (!$add['CRM_ID']) return;

    if (!array_key_exists('PROBLEM_ID', $add))
      $add['PROBLEM_ID'] = $this->db->nextId ('problem_id');

    $this->Set_Audit_Info ($add);

    $to_date = to_date();
    $h = $this->db->prepare ("
      insert into problem_type (
        problem_type,
        crm_id,
        updated_by,
        created_by,
        updated_date,
        created_date,
        problem_id
      ) values (
        ?, ?, ?, ?,
	$to_date,
	$to_date,
        ?
      )
    ");
    $ok = $this->db->execute ($h, array(
      $add['PROBLEM_TYPE'],
      $add['CRM_ID'],
      $add['UPDATED_BY'],
      $add['CREATED_BY'],
      $add['UPDATED_DATE'],
      $add['CREATED_DATE'],
      $add['PROBLEM_ID']
    ));
    if (DB::isError($ok)) {
       $this->error = 'Problem_Type::Add(): ' . $ok->getMessage()
	. $ok->getUserInfo();
       return;
    }

    return $add['PROBLEM_ID'];
  }

  function Delete ($ptype, $crm_id='') {
    if (!$crm_id) $crm_id = $this->Current_CRM ();
    if (!$crm_id) return 0;

    $h = $this->db->prepare ('
      delete from problem_type
       where problem_id = ?
         and crm_id = ?
    ');
    $ok = $this->db->execute ($h, array($ptype, $crm_id));

    if (DB::isError($ok)) {
	$this->error = 'Problem_Type::Delete(): ' . $ok->getMessage()
	    . $ok->getUserInfo();
	return;
    }

    return 1;
  }

  function is_Valid ($prid, $crm_id='') {
    if (!$crm_id) $crm_id = $this->Current_CRM ();
    if (!$crm_id) return 0;

    $h = $this->db->prepare ("
      select count(*) 
        from problem_type
       where problem_id = ?
         and crm_id = ?
    ");
    $res = $this->db->execute ($h, $prid, $crm_id);
    if (DB::isError($res)) {
       $this->error = 'Problem_Type::is_Valid(): '
	    . $res->getMessage() . $res->getUserInfo();
       return;
    }
    list ($c) = $res->fetchRow ();

    return $c;
  }

  function &List_All ($in_drop='', $crm_id='') {
    if (!$crm_id) $crm_id = $this->Current_CRM ();
    if (!$crm_id) return 0;

    $binds = array ($crm_id);

    $sql = '
      select problem_id,
             problem_type,
             in_drop_downs
        from problem_type
       where crm_id = ?
    ';

    if ($in_drop) {
      $sql .= ' and in_drop_downs = ? ';
      $binds[] = $in_drop;
    }

    $sql .= ' order by upper(problem_type) ';

    $res = $this->db->getAll($sql, $binds);

    if (DB::isError($res)) {
	$this->error = 'Problem_Type::List_All(): ' . $res->getMessage() .
	    $res->getUserInfo();
	return;
    }

    return $res;
  }

  function Move ($prid, $to_ptype, $crm_id='') {
    if (!$crm_id) $crm_id = $this->Current_CRM ();
    if (!$crm_id) return 0;

    $add = array();
    $this->Set_Audit_Info( $add );

    $to_date = to_date();
    $upex = $this->db->prepare ("
      update ticket_crm_map
         set problem_id = ?,
             updated_by = ?,
             updated_date = $to_date
       where problem_id = ?
         and crm_id = ?
    ");
    $ok = $this->db->execute ($upex, array($to_ptype, $add['UPDATED_BY'], $add['UPDATED_DATE'], $prid, $crm_id));

    if (DB::isError($ok)) {
      $this->error = 'Problem_Type::Move(): ' . $ok->getMessage() . "\n"
	. $ok->getUserInfo();
      return;
    }

    return true;
  }

  function Select_by_Problem_Type ($ptype) {
    $problem_type_r = new Problem_Type_Record ();
    $problem_type_r->Select_by_Problem_Type ($ptype);
    return $problem_type_r;
  }

  function Update () {

    $this->Set_Audit_Info( $this );

    $to_date = to_date();
    $h = $this->db->prepare ("
      update problem_type
         set in_drop_downs = ?,
             problem_type = ?,
             updated_by = ?,
             updated_date = $to_date
       where problem_id = ?
         and crm_id = ?
    ");
    $res = $this->db->execute ($h, array(
      $this->in_drop_downs,
      $this->problem_type,
      $this->updated_by,
      $this->updated_date,
      $this->problem_id,
      $this->crm_id
    ));
    if (DB::isError($res)) {
       $this->error = $res->getMessage();
       return 0;
    }

    return 0;
  }
}

class Problem_Type_Record extends Problem_Type {
  function Problem_Type_Record () {
    global $dbi;
    $this->db = &$dbi;
  }

  function Select_by_Problem_Type ($ptype, $crm_id='') {
    if (!$crm_id) $crm_id = $this->Current_CRM ();
    if (!$crm_id) return 0;

    $h = $this->db->prepare ('
      select problem_type,
             in_drop_downs,
             crm_id,
             problem_id
        from problem_type
       where problem_id = ?
         and crm_id = ?
    ');
    $res = $this->db->execute( $h, array($ptype,$crm_id) );
    $r = $res->fetchRow (DB_FETCHMODE_ASSOC);

    foreach ($r as $k => $v) {
      $k = strtolower ($k);
      $this->$k = $v;
    }
  }
}
?>
